package com.pfe.www;

import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.LinkedList;
import java.util.Collections;
import java.util.Comparator;

public class SMSWordsCounter {

    TreeMap<String, Integer> map;
    Locale loc;

    public SMSWordsCounter() {
        map = new TreeMap<String, Integer>();
        loc = Locale.getDefault();
    }

    public void parseText(String input){
        String[] words = input.split("[ \n\t\r.,;':!?(){}]");
        String key ;
        int value;

        for (int counter = 0; counter < words.length; counter++) {
            key = words[counter].toLowerCase(Locale.FRANCE);

            // Keep only words with more than 3 letters
            if (key.length() > 3) {
                if (map.get(key) == null) {
                    map.put(key, 1);
                } else {
                    value = map.get(key).intValue();
                    value++;
                    map.put(key, value);
                }
            }
        }
    }

    // Get the most used words (nb_words words)
    public String[] getBestWordsByNumberWords(int nb_words){
        LinkedList<String> sortedList = getSortedList();
        String[] words = new String[Math.min(nb_words, sortedList.size())];

        int ind = 0;
        for (String s : sortedList) {
            words[ind] = s;
            ind++;
            if(ind == nb_words)
                break;
        }
        return words;
    }
    
    // Get the words used more than nb_time time (pertinent if there are a lot of SMS)
    public String[] getBestWordsByNumberTime(int nb_time) {
    	String [] words = new String[map.size()];
        Set<Map.Entry<String, Integer>> entrySet = map.entrySet();

        int ind = 0;
        for (Map.Entry<String, Integer> entry : entrySet) {
        	if(entry.getValue() > nb_time) {
        		words[ind] = (String)entry.getKey();
                ind ++;
        	}
        }
        
        ind --;
        String out[] = new String[ind];
        for (int i = 0; i < ind; i++) {
			out[i] = words[i];
		}
        return out;
    }

    public LinkedList<String> getSortedList() {
        LinkedList<Map.Entry<String,Integer>> list = new LinkedList<Map.Entry<String,Integer>>( map.entrySet() );
        Collections.sort(list, new Comparator<Map.Entry<String,Integer>>() {
            @Override
            public int compare(Map.Entry<String,Integer> o1, Map.Entry<String,Integer> o2) {
                return (o1.getValue()).compareTo(o2.getValue());
            }
        });

        LinkedList<String> result = new LinkedList<String>();
        for(Map.Entry<String,Integer> entry : list) {
            result.add(entry.getKey());
        }
        Collections.reverse(result);
        return result;
    }
}
